<?php
/**
 * Advanced Notifications Database helper class.
 *
 * @package WC_Advanced_Notifications
 */

namespace WooCommerce\AdvancedNotifications;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * DB_Helper class.
 */
class DB_Helper {

	/**
	 * Retrieve all notifications.
	 *
	 * @return array
	 */
	public static function get_all_notifications() {
		global $wpdb;

		return $wpdb->get_results( "SELECT * FROM {$wpdb->prefix}advanced_notifications" );
	}

	/**
	 * Retrieve triggers for a specific product.
	 *
	 * @param int $product_id Product ID.
	 *
	 * @return array
	 */
	public static function get_product_triggers( $product_id ) {
		global $wpdb;

		return $wpdb->get_col(
			$wpdb->prepare(
				"SELECT notification_id FROM {$wpdb->prefix}advanced_notification_triggers WHERE object_id = %d AND object_type = 'product';",
				absint( $product_id )
			)
		);
	}

	/**
	 * Delete current triggers for a specific product.
	 *
	 * @param int|string $product_id Post ID.
	 * @param array      $recipients Array of recipient IDs.
	 */
	public static function delete_product_triggers( $product_id, array $recipients ) {
		global $wpdb;

		if ( empty( $recipients ) ) {
			return;
		}

		// Ensure each recipient is an integer and prepare placeholders.
		$placeholders = implode( ',', array_fill( 0, count( $recipients ), '%d' ) );

		// Combine $product_id with $recipients array for the final prepared statement.
		$wpdb->query(
			$wpdb->prepare(
				"DELETE FROM {$wpdb->prefix}advanced_notification_triggers
				WHERE object_id = %d
				AND object_type = 'product'
				AND object_id NOT IN ( $placeholders )", // phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared
				array_merge( array( absint( $product_id ) ), array_map( 'absint', $recipients ) )
			)
		);
	}

	/**
	 * Insert new triggers into the database.
	 *
	 * @param array $triggers The triggers to insert.
	 */
	public static function insert_product_notification_triggers( array $triggers ) {
		global $wpdb;

		if ( empty( $triggers ) ) {
			return;
		}

		foreach ( $triggers as $trigger ) {
			if ( isset( $trigger['notification_id'], $trigger['object_id'], $trigger['object_type'] ) ) {
				$wpdb->insert(
					"{$wpdb->prefix}advanced_notification_triggers",
					array(
						'notification_id' => $trigger['notification_id'],
						'object_id'       => $trigger['object_id'],
						'object_type'     => $trigger['object_type'],
					),
					array( '%d', '%d', '%s' )
				);
			}
		}
	}

	/**
	 * Delete current triggers for a specific recipient.
	 *
	 * @param int $recipient_id Recipient ID.
	 */
	public static function recipient_notification_triggers( $recipient_id ) {
		global $wpdb;

		$wpdb->query(
			$wpdb->prepare(
				"
				DELETE FROM {$wpdb->prefix}advanced_notification_triggers
				WHERE notification_id = %d
				AND object_type != 'product';
				",
				absint( $recipient_id )
			)
		);
	}

	/**
	 * Insert notification triggers for a recipient.
	 *
	 * @param int   $notification_id Notification ID.
	 * @param array $posted_triggers Triggers to insert.
	 *
	 * @return void
	 */
	public static function insert_recipient_notification_triggers( $notification_id, $posted_triggers ) {
		global $wpdb;

		foreach ( $posted_triggers as $trigger ) {
			if ( 'all' === $trigger ) {
				$data = array(
					'notification_id' => $notification_id,
					'object_id'       => 0,
					'object_type'     => '',
				);
			} else {
				$trigger_parts = explode( ':', $trigger );
				$term          = esc_attr( $trigger_parts[0] );
				$id            = absint( $trigger_parts[1] );

				$data = array(
					'notification_id' => $notification_id,
					'object_id'       => $id,
					'object_type'     => $term,
				);
			}

			$wpdb->insert(
				"{$wpdb->prefix}advanced_notification_triggers",
				$data,
				array( '%d', '%d', '%s' )
			);
		}
	}
}
