<?php
/**
 * Advanced Notifications Product Editor class.
 *
 * @package WC_Advanced_Notifications
 */

namespace WooCommerce\AdvancedNotifications;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

require_once WC_ADVANCED_NOTIFICATIONS_ABSPATH . 'includes/class-db-helper.php';
require_once WC_ADVANCED_NOTIFICATIONS_ABSPATH . 'includes/class-rest-controller.php';

use Automattic\WooCommerce\Admin\Features\ProductBlockEditor\BlockRegistry;
use Automattic\WooCommerce\Admin\BlockTemplates\BlockInterface;
use Automattic\WooCommerce\Utilities\FeaturesUtil;

/**
 * Product_Editor class.
 */
class Product_Editor {

	/**
	 * __construct function.
	 */
	public function __construct() {
		// Declaring compatibility with the product editor.
		add_action( 'before_woocommerce_init', array( $this, 'declare_product_editor_compatibility' ) );
		add_filter( 'woocommerce_rest_api_get_rest_namespaces', array( $this, 'add_rest_controller' ) );

		add_action( 'init', array( $this, 'extension_advanced_notifications_block_init' ) );
		add_action( 'woocommerce_block_template_area_product-form_after_add_block_product-summary', array( $this, 'add_notifications_fields' ) );
		add_filter( 'woocommerce_rest_pre_insert_product_object', array( $this, 'save_advanced_notifications_data' ) );
	}

	/**
	 * Declare Product Editor compatibility.
	 *
	 * @return void
	 */
	public function declare_product_editor_compatibility() {
		if ( class_exists( FeaturesUtil::class ) ) {
			FeaturesUtil::declare_compatibility( 'product_block_editor', WC_ADVANCED_NOTIFICATIONS_BASENAME, true );
		}
	}

	/**
	 * Registers the block using the metadata loaded from the `block.json` file.
	 * Behind the scenes, it registers also all assets so they can be enqueued
	 * through the block editor in the corresponding context.
	 *
	 * @see https://developer.wordpress.org/reference/functions/register_block_type/
	 *
	 * @return void
	 */
	public function extension_advanced_notifications_block_init() {
		// phpcs:ignore WordPress.Security.NonceVerification.Recommended --- No need to use nonce as no DB operation required.
		if ( isset( $_GET['page'] ) && 'wc-admin' === $_GET['page'] ) {
			BlockRegistry::get_instance()->register_block_type_from_metadata( WC_ADVANCED_NOTIFICATIONS_ABSPATH . '/dist/advanced-notifications' );
		}
	}

	/**
	 * Add custom fields to the product editor general section.
	 *
	 * @param BlockInterface $product_sale_tax The product sale tax block.
	 *
	 * @return void
	 */
	public function add_notifications_fields( BlockInterface $product_sale_tax ) {
		$parent = $product_sale_tax->get_parent();
		$parent->add_block(
			array(
				'id'         => 'notification_recipients',
				'order'      => 20,
				'blockName'  => 'extension/woocommerce-advanced-notifications',
				'attributes' => array(
					'label' => esc_html__( 'Notifications', 'woocommerce-advanced-notifications' ),
					'help'  => esc_attr__( 'Choose recipients for this product.', 'woocommerce-advanced-notifications' ),
				),
			)
		);
	}

	/**
	 * Add Per-Product data to REST request.
	 *
	 * @param \WC_Product $product  Product object.
	 *
	 * @return \WC_Product Modified product object.
	 */
	public function save_advanced_notifications_data( $product ) {
		$product_notifications = $product->get_meta( '_advanced_notifications_triggers' );
		$recipients            = array();
		$triggers              = array();

		foreach ( $product_notifications as $notification ) {
			if ( ! $notification['selected'] ) {
				continue;
			}

			$recipient    = absint( $notification['value'] );
			$recipients[] = $recipient;
			$triggers[]   = array(
				'notification_id' => $recipient,
				'object_id'       => $product->get_id(),
				'object_type'     => 'product',
			);
		}

		// Delete current triggers for this product.
		DB_Helper::delete_product_triggers( $product->get_id(), $recipients );

		// Save new.
		DB_Helper::insert_product_notification_triggers( $triggers );

		return $product;
	}

	/**
	 * Add plugin custom WC REST API endpoint.
	 *
	 * @param array $controllers List of WooCommerce controllers.
	 *
	 * @return array Modified list of WooCommerce controllers.
	 */
	public function add_rest_controller( $controllers ) {
		$controllers['wc/v3']['product_notifications'] = 'WooCommerce\AdvancedNotifications\REST_Controller';

		return $controllers;
	}
}
