<?php
/**
 * Advanced Notifications Custom REST class.
 *
 * @package WC_Advanced_Notifications
 */

namespace WooCommerce\AdvancedNotifications;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Advanced Notifications Custom REST class.
 */
class REST_Controller {
	/**
	 * WooCommerce REST API namespace.
	 *
	 * @var string
	 */
	protected string $namespace = 'wc/v3';

	/**
	 * This plugin REST base.
	 *
	 * @var string
	 */
	protected string $rest_base = 'product_notifications/(?P<product_id>\d+)';

	/**
	 * Registering the custom routes.
	 *
	 * @return void
	 */
	public function register_routes() {
		register_rest_route(
			$this->namespace,
			'/' . $this->rest_base,
			array(
				'methods'             => \WP_REST_Server::READABLE,
				'callback'            => array( $this, 'get_product_notifications_trigger' ),
				'permission_callback' => array( $this, 'get_notifications_permissions_check' ),
			)
		);
	}

	/**
	 * Retrieve triggers for a specific product.
	 *
	 * @param \WP_REST_Request $request Request data.
	 *
	 * @return array|\WP_Error
	 */
	public function get_product_notifications_trigger( \WP_REST_Request $request ) {
		$product_id = (int) $request['product_id'];

		if ( 'product' !== get_post_type( $product_id ) ) {
			return new \WP_Error( 'woocommerce_rest_product_invalid_id', __( 'Invalid product ID.', 'woocommerce-advanced-notifications' ), array( 'status' => 404 ) );
		}

		$notifications = DB_Helper::get_all_notifications();
		$triggers      = DB_Helper::get_product_triggers( $product_id );
		$options       = array(
			'notifications' => array(),
		);

		foreach ( $notifications as $notification ) {
			$options['notifications'][] = array(
				'label'    => esc_html( $notification->recipient_name ),
				'value'    => esc_attr( $notification->notification_id ),
				'selected' => in_array( $notification->notification_id, $triggers, true ),
			);
		}

		return $options;
	}

	/**
	 * Makes sure the current user has access to WRITE the settings APIs.
	 *
	 * @return \WP_Error|bool
	 */
	public function get_notifications_permissions_check() {
		// phpcs:ignore --- WooCommerce custom capability.
		if ( ! current_user_can( 'edit_products' ) ) {
			return new \WP_Error( 'woocommerce_rest_cannot_edit', __( 'Sorry, you cannot read this resource.', 'woocommerce-advanced-notifications' ), array( 'status' => rest_authorization_required_code() ) );
		}

		return true;
	}
}
