<?php
/**
 * Advanced Notifications for admin UI class.
 *
 * @package WC_Advanced_Notifications
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

require_once WC_ADVANCED_NOTIFICATIONS_ABSPATH . 'includes/class-db-helper.php';

use WooCommerce\AdvancedNotifications\DB_Helper;

/**
 * WC_Advanced_Notifications_Admin class.
 */
class WC_Advanced_Notifications_Admin {

	/**
	 * Current notification that being edited.
	 *
	 * @var object
	 */
	private $editing;

	/**
	 * Current notification ID that being edited.
	 *
	 * @var int
	 */
	private $editing_id;

	/**
	 * __construct function.
	 */
	public function __construct() {
		// Admin menu.
		add_action( 'admin_menu', array( $this, 'admin_menu' ) );
		add_action( 'woocommerce_screen_ids', array( $this, 'screen_ids' ) );
		add_filter( 'set-screen-option', array( $this, 'set_screen_options' ), 10, 3 );
		add_filter( 'manage_woocommerce_page_advanced-notifications_columns', array( $this, 'pass_custom_columns' ) );

		// Meta.
		add_action( 'woocommerce_product_options_general_product_data', array( $this, 'write_panel' ) );
		add_action( 'woocommerce_process_product_meta', array( $this, 'write_panel_save' ) );
	}

	/**
	 * Screen ids.
	 *
	 * @param array $ids Screen IDs.
	 *
	 * @return array
	 */
	public function screen_ids( $ids ) {
		$wc_screen_id = strtolower( __( 'WooCommerce', 'woocommerce-advanced-notifications' ) );

		$ids[] = $wc_screen_id . '_page_advanced-notifications';

		return $ids;
	}

	/**
	 * Adding more menu in WP Admin menu.
	 */
	public function admin_menu() {
		$page = add_submenu_page( 'woocommerce', __( 'Advanced Notifications', 'woocommerce-advanced-notifications' ), __( 'Notifications', 'woocommerce-advanced-notifications' ), 'manage_woocommerce', 'advanced-notifications', array( $this, 'admin_screen' ) ); // phpcs:ignore WordPress.WP.Capabilities.Unknown --- `manage_woocommerce` is a native role from WooCommerce

		add_action( "load-$page", array( $this, 'screen_options' ) );
		if ( function_exists( 'woocommerce_admin_css' ) ) {
			add_action( 'admin_print_styles-' . $page, 'woocommerce_admin_css' );
		}
		add_action( 'admin_print_styles-' . $page, array( $this, 'admin_enqueue' ) );
	}

	/**
	 * Admin enqueue function.
	 */
	public function admin_enqueue() {
		wp_enqueue_style( 'notifications_css', plugins_url( 'assets/css/admin.css', __DIR__ ), array(), WC_ADVANCED_NOTIFICATIONS_VERSION );
	}

	/**
	 * Screen options for the advanced notifications page.
	 */
	public function screen_options() {
		add_screen_option(
			'per_page',
			array(
				'default' => 25,
				'option'  => 'woocommerce_advanced_notifications_per_page',
			)
		);
	}

	/**
	 * Saves our custom screen options to the database
	 *
	 * @param string $status Screen status.
	 * @param string $option Option ID.
	 * @param mixed  $value Option value.
	 *
	 * @return string.
	 */
	public function set_screen_options( $status, $option, $value ) {
		if ( 'woocommerce_advanced_notifications_per_page' === $option ) {
			return $value;
		}
		return $status;
	}

	/**
	 * Tells the screen options which columns we can hide.
	 */
	public function pass_custom_columns() {
		return array(
			'notification_type'       => esc_html__( 'Notification Types', 'woocommerce-advanced-notifications' ),
			'notification_triggers'   => esc_html__( 'Triggers', 'woocommerce-advanced-notifications' ),
			'notification_plain_text' => esc_html__( 'Plain text?', 'woocommerce-advanced-notifications' ),
			'notification_prices'     => esc_html__( 'Prices?', 'woocommerce-advanced-notifications' ),
			'notification_totals'     => esc_html__( 'Totals?', 'woocommerce-advanced-notifications' ),
			'notification_sent_count' => esc_html__( 'Sent count', 'woocommerce-advanced-notifications' ),
		);
	}

	/**
	 * Add new option on the product page.
	 */
	public function write_panel() {
		global $post;

		$notifications = DB_Helper::get_all_notifications();

		if ( ! $notifications ) {
			return;
		}

		echo '<div class="options_group">';

		$triggers = DB_Helper::get_product_triggers( absint( $post->ID ) );
		?>
		<p class="form-field">
			<label><?php esc_html_e( 'Notifications', 'woocommerce-advanced-notifications' ); ?></label>
			<select id="notification_recipients" name="notification_recipients[]" multiple="multiple" style="width:300px;" data-placeholder="<?php esc_attr_e( 'Choose recipients for this product&hellip;', 'woocommerce-advanced-notifications' ); ?>" class="wc-enhanced-select">
				<?php
				foreach ( $notifications as $notification ) {
					echo '<option value="' . esc_attr( $notification->notification_id ) . '" ' . selected( in_array( $notification->notification_id, $triggers, true ), true, false ) . '>' . esc_html( $notification->recipient_name ) . '</option>';
				}
				?>
			</select>
		</p>
		<?php

		echo '</div>';
	}

	/**
	 * Save the new product options.
	 *
	 * @param int $post_id Post ID.
	 */
	public function write_panel_save( $post_id ) {
		$recipients = array( 0 );
		$triggers   = array();

		// Get new notification recipients.
		// No need to be verified as it has been verified on WC_Meta_Box_Product_Data::save().
		// It's sanitized later using `absint()` on line 139.
		$notification_recipients = isset( $_POST['notification_recipients'] ) ? $_POST['notification_recipients'] : ''; // phpcs:ignore

		if ( is_array( $notification_recipients ) ) {
			foreach ( $notification_recipients as $recipient ) {
				$recipient    = absint( $recipient );
				$recipients[] = $recipient;
				$triggers[]   = array(
					'notification_id' => $recipient,
					'object_id'       => $post_id,
					'object_type'     => 'product',
				);
			}
		}

		// Delete current triggers for this product.
		DB_Helper::delete_product_triggers( $post_id, $recipients );

		// Save new.
		DB_Helper::insert_product_notification_triggers( $triggers );
	}

	/**
	 * Display the admin UI.
	 */
	public function admin_screen() {
		global $wpdb;

		$admin = $this;

		if ( ! empty( $_GET['delete'] ) ) {

			check_admin_referer( 'delete_notification' );

			$delete = absint( $_GET['delete'] );

			$wpdb->query(
				$wpdb->prepare( "DELETE FROM {$wpdb->prefix}advanced_notifications WHERE notification_id = %d;", $delete )
			);
			$wpdb->query(
				$wpdb->prepare( "DELETE FROM {$wpdb->prefix}advanced_notification_triggers WHERE notification_id = %d;", $delete )
			);

			echo '<div class="updated fade"><p>' . esc_html__( 'Notification deleted successfully', 'woocommerce-advanced-notifications' ) . '</p></div>';

		} elseif ( ! empty( $_GET['add'] ) ) {

			if ( ! empty( $_POST['save_recipient'] ) ) {

				check_admin_referer( 'woocommerce_save_recipient' );

				$result = $this->add_recipient();

				if ( is_wp_error( $result ) ) {
					echo '<div class="error"><p>' . esc_html( $result->get_error_message() ) . '</p></div>';
				} elseif ( $result ) {
					echo '<div class="updated fade"><p>' . esc_html__( 'Notification saved successfully', 'woocommerce-advanced-notifications' ) . '</p></div>';
				}
			}

			include_once WC_ADVANCED_NOTIFICATIONS_ABSPATH . 'includes/views/admin-screen-edit.php';
			return;

		} elseif ( ! empty( $_GET['edit'] ) ) {

			$this->editing_id = absint( $_GET['edit'] );
			$this->editing    = $wpdb->get_row(
				$wpdb->prepare(
					"SELECT * FROM {$wpdb->prefix}advanced_notifications WHERE notification_id = %d;",
					$this->editing_id
				)
			);

			if ( ! empty( $_POST['save_recipient'] ) ) {

				check_admin_referer( 'woocommerce_save_recipient' );

				$result = $this->save_recipient();

				if ( is_wp_error( $result ) ) {
					echo '<div class="error"><p>' . esc_html( $result->get_error_message() ) . '</p></div>';
				} elseif ( $result ) {

					echo '<div class="updated fade"><p>' . esc_html__( 'Notification saved successfully', 'woocommerce-advanced-notifications' ) . '</p></div>';
				}
			}

			include_once WC_ADVANCED_NOTIFICATIONS_ABSPATH . 'includes/views/admin-screen-edit.php';
			return;
		}

		if ( ! empty( $_GET['success'] ) ) {
			echo '<div class="updated fade"><p>' . esc_html__( 'Notification saved successfully', 'woocommerce-advanced-notifications' ) . '</p></div>';
		}

		if ( ! empty( $_GET['deleted'] ) ) {
			echo '<div class="updated fade"><p>' . esc_html__( 'Notification deleted successfully', 'woocommerce-advanced-notifications' ) . '</p></div>';
		}

		if ( ! class_exists( 'WP_List_Table' ) ) {
			include_once ABSPATH . 'wp-admin/views/class-wp-list-table.php';
		}
		include_once WC_ADVANCED_NOTIFICATIONS_ABSPATH . 'includes/class-wc-advanced-notifications-table.php';
		include_once WC_ADVANCED_NOTIFICATIONS_ABSPATH . 'includes/views/admin-screen.php';
	}


	/**
	 * Get field value
	 *
	 * @param string $name Name of the field.
	 *
	 * @return mixed
	 */
	public function field_value( $name ) {
		global $wpdb;

		$value = '';

		if ( isset( $this->editing->$name ) ) {

			$value = $this->editing->$name;

		} elseif ( 'notification_triggers' === $name ) {

			$value = $wpdb->get_col(
				$wpdb->prepare(
					"SELECT object_id FROM {$wpdb->prefix}advanced_notification_triggers WHERE notification_id = %d;",
					absint( $this->editing_id )
				)
			);

		}

		// It has been verified on WC_Advanced_Notifications_Admin::admin_screen() and will be sanitized later on line 266.
		$value = isset( $_POST[ $name ] ) ? $_POST[ $name ] : maybe_unserialize( $value ); // phpcs:ignore

		if ( is_array( $value ) ) {
			$value = array_map( 'trim', array_map( 'esc_attr', array_map( 'stripslashes', $value ) ) );
		} else {
			$value = trim( esc_attr( stripslashes( $value ) ) );
		}

		return $value;
	}

	/**
	 * Add recipient to the notification.
	 */
	public function add_recipient() {
		global $wpdb;

		// phpcs:disable WordPress.Security.NonceVerification.Missing --- No need to verify as it has been verified on WC_Advanced_Notifications_Admin::admin_screen().
		$recipient_name          = isset( $_POST['recipient_name'] ) ? sanitize_text_field( wp_unslash( $_POST['recipient_name'] ) ) : '';
		$recipient_email         = isset( $_POST['recipient_email'] ) ? sanitize_email( wp_unslash( $_POST['recipient_email'] ) ) : '';
		$recipient_address       = isset( $_POST['recipient_address'] ) ? sanitize_text_field( wp_unslash( $_POST['recipient_address'] ) ) : '';
		$recipient_phone         = isset( $_POST['recipient_phone'] ) ? sanitize_text_field( wp_unslash( $_POST['recipient_phone'] ) ) : '';
		$recipient_website       = isset( $_POST['recipient_website'] ) ? esc_url_raw( wp_unslash( $_POST['recipient_website'] ) ) : '';
		$notification_type       = isset( $_POST['notification_type'] ) ? array_filter(
			wc_clean( wp_unslash( $_POST['notification_type'] ) ), // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized --- It has been sanitized using wc_clean().
			function ( $val ) {
				return ! empty( $val );
			}
		) : array();
		$notification_plain_text = ! empty( $_POST['notification_plain_text'] ) ? 1 : 0;
		$notification_totals     = ! empty( $_POST['notification_totals'] ) ? 1 : 0;
		$notification_prices     = ! empty( $_POST['notification_prices'] ) ? 1 : 0;
		$notification_all_items  = ! empty( $_POST['notification_include_all_items'] ) ? 1 : 0;
		// phpcs:enable WordPress.Security.NonceVerification.Missing

		// Validate.
		if ( empty( $recipient_name ) ) {
			return new WP_Error( 'input', __( 'Recipient name is a required field', 'woocommerce-advanced-notifications' ) );
		}

		if ( empty( $recipient_email ) ) {
			return new WP_Error( 'input', __( 'Recipient email is a required field', 'woocommerce-advanced-notifications' ) );
		}

		$recipient_emails = array_map( 'trim', explode( ',', $recipient_email ) );

		foreach ( $recipient_emails as $email ) {
			if ( ! is_email( $email ) ) {
				return new WP_Error( 'input', __( 'A recipient email is invalid:', 'woocommerce-advanced-notifications' ) . ' ' . $email );
			}
		}

		// Insert.
		$result = $wpdb->insert(
			"{$wpdb->prefix}advanced_notifications",
			array(
				'recipient_name'                 => $recipient_name,
				'recipient_email'                => $recipient_email,
				'recipient_address'              => $recipient_address,
				'recipient_phone'                => $recipient_phone,
				'recipient_website'              => $recipient_website,
				'notification_plain_text'        => $notification_plain_text,
				'notification_type'              => serialize( $notification_type ), // phpcs:ignore WordPress.PHP.DiscouragedPHPFunctions.serialize_serialize --- Need to use `serialize()` to save the array value.
				'notification_totals'            => $notification_totals,
				'notification_prices'            => $notification_prices,
				'notification_include_all_items' => $notification_all_items,
			),
			array(
				'%s',
				'%s',
				'%s',
				'%s',
				'%s',
				'%d',
				'%s',
				'%d',
				'%d',
				'%d',
			)
		);

		$notification_id = $wpdb->insert_id;

		if ( $result && $notification_id ) {
			// Store triggers. No need to verify as it has been verified on WC_Advanced_Notifications_Admin::admin_screen().
			$posted_triggers = isset( $_POST['notification_triggers'] ) ? array_filter( array_map( 'esc_attr', array_map( 'trim', (array) $_POST['notification_triggers'] ) ) ) : array(); // phpcs:ignore

			DB_Helper::insert_recipient_notification_triggers( $notification_id, $posted_triggers );

			return true;
		}

		return false;
	}

	/**
	 * Save recipient into notification record.
	 */
	public function save_recipient() {
		global $wpdb;

		// No need to verify as it has been verified on WC_Advanced_Notifications_Admin::admin_screen().
		// phpcs:disable WordPress.Security.NonceVerification.Missing
		$recipient_name          = isset( $_POST['recipient_name'] ) ? sanitize_text_field( wp_unslash( $_POST['recipient_name'] ) ) : '';
		$recipient_email         = isset( $_POST['recipient_email'] ) ? sanitize_text_field( wp_unslash( $_POST['recipient_email'] ) ) : '';
		$recipient_address       = isset( $_POST['recipient_address'] ) ? sanitize_text_field( wp_unslash( $_POST['recipient_address'] ) ) : '';
		$recipient_phone         = isset( $_POST['recipient_phone'] ) ? sanitize_text_field( wp_unslash( $_POST['recipient_phone'] ) ) : '';
		$recipient_website       = isset( $_POST['recipient_website'] ) ? sanitize_text_field( wp_unslash( $_POST['recipient_website'] ) ) : '';
		$notification_type       = isset( $_POST['notification_type'] ) ? array_filter(
			wc_clean( wp_unslash( $_POST['notification_type'] ) ), // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized --- It has been sanitized using wc_clean().
			function ( $val ) {
				return ! empty( $val );
			}
		) : array();
		$notification_plain_text = ! empty( $_POST['notification_plain_text'] ) ? 1 : 0;
		$notification_totals     = ! empty( $_POST['notification_totals'] ) ? 1 : 0;
		$notification_prices     = ! empty( $_POST['notification_prices'] ) ? 1 : 0;
		$notification_all_items  = ! empty( $_POST['notification_include_all_items'] ) ? 1 : 0;
		// phpcs:enable WordPress.Security.NonceVerification.Missing

		// Validate.
		if ( empty( $recipient_name ) ) {
			return new WP_Error( 'input', __( 'Recipient name is a required field', 'woocommerce-advanced-notifications' ) );
		}

		if ( empty( $recipient_email ) ) {
			return new WP_Error( 'input', __( 'Recipient email is a required field', 'woocommerce-advanced-notifications' ) );
		}

		$recipient_emails = array_map( 'trim', explode( ',', $recipient_email ) );

		foreach ( $recipient_emails as $email ) {
			if ( ! is_email( $email ) ) {
				return new WP_Error( 'input', __( 'A recipient email is invalid:', 'woocommerce-advanced-notifications' ) . ' ' . $email );
			}
		}

		// Insert recipient.
		$wpdb->update(
			"{$wpdb->prefix}advanced_notifications",
			array(
				'recipient_name'                 => $recipient_name,
				'recipient_email'                => $recipient_email,
				'recipient_address'              => $recipient_address,
				'recipient_phone'                => $recipient_phone,
				'recipient_website'              => $recipient_website,
				'notification_plain_text'        => $notification_plain_text,
				'notification_type'              => serialize( $notification_type ), // phpcs:ignore WordPress.PHP.DiscouragedPHPFunctions.serialize_serialize --- Need to use `serialize()` to save the array value.
				'notification_totals'            => $notification_totals,
				'notification_prices'            => $notification_prices,
				'notification_include_all_items' => $notification_all_items,
			),
			array( 'notification_id' => absint( $this->editing_id ) ),
			array(
				'%s',
				'%s',
				'%s',
				'%s',
				'%s',
				'%d',
				'%s',
				'%d',
				'%d',
				'%d',
			),
			array( '%d' )
		);

		// Delete old triggers.
		DB_Helper::recipient_notification_triggers( absint( $this->editing_id ) );

		// Store triggers. No need to verify as it has been verified on WC_Advanced_Notifications_Admin::admin_screen().
		$posted_triggers = isset( $_POST['notification_triggers'] ) ? array_filter( array_map( 'esc_attr', array_map( 'trim', (array) $_POST['notification_triggers'] ) ) ) : array(); // phpcs:ignore

		DB_Helper::insert_recipient_notification_triggers( $this->editing_id, $posted_triggers );

		return true;
	}
}
