<?php
/**
 * Plugin Name: WooCommerce Advanced Notifications
 * Plugin URI: https://woocommerce.com/products/advanced-notifications
 * Description: Add additonal, advanced order and stock notifications to WordPress - ideal for improving store management or for dropshippers.
 * Version: 1.5.4
 * Author: WooCommerce
 * Author URI: https://woocommerce.com
 * Text Domain: woocommerce-advanced-notifications
 * Domain Path: /languages/
 * Requires Plugins: woocommerce
 * Requires PHP: 7.4
 * Requires at least: 6.6
 * Tested up to: 6.7
 * WC requires at least: 9.6
 * WC tested up to: 9.8
 * Woo: 18740:112372c44b002fea2640bd6bfafbca27
 *
 * Copyright: © 2025 WooCommerce
 * License: GNU General Public License v3.0
 * License URI: https://www.gnu.org/licenses/gpl-3.0.html
 *
 * @package WC_Advanced_Notifications
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

define( 'WC_ADVANCED_NOTIFICATIONS_VERSION', '1.5.4' ); // WRCS: DEFINED_VERSION.
define( 'WC_ADVANCED_NOTIFICATIONS_DB_VERSION', '1.0.1' ); // DB Version.
define( 'WC_ADVANCED_NOTIFICATIONS_FILE', __FILE__ ); // Plugin main file.
define( 'WC_ADVANCED_NOTIFICATIONS_ABSPATH', trailingslashit( __DIR__ ) ); // Plugin main folder.
define( 'WC_ADVANCED_NOTIFICATIONS_BASENAME', plugin_basename( WC_ADVANCED_NOTIFICATIONS_FILE ) );


/**
 * Init advanced notifications function.
 */
function init_advanced_notifications() {
	add_action( 'after_setup_theme', 'woocommerce_advanced_notifications_load_textdomain' );

	if ( class_exists( 'WooCommerce' ) ) {
		woocommerce_advanced_notifications_db_check();
		add_filter( 'woocommerce_translations_updates_for_' . basename( WC_ADVANCED_NOTIFICATIONS_FILE, '.php' ), '__return_true' );
		add_action( 'before_woocommerce_init', 'woocommerce_advanced_notifications_loader' );
	} else {
		add_action( 'admin_notices', 'woocommerce_advanced_notifications_woocommerce_deactivated' );
	}
}
add_action( 'plugins_loaded', 'init_advanced_notifications' );

/**
 * Plugin loader.
 */
function woocommerce_advanced_notifications_loader() {
	include_once WC_ADVANCED_NOTIFICATIONS_ABSPATH . 'includes/class-wc-advanced-notifications.php';
	include_once WC_ADVANCED_NOTIFICATIONS_ABSPATH . 'includes/class-wc-advanced-notifications-privacy.php';
}

/**
 * Load localization.
 */
function woocommerce_advanced_notifications_load_textdomain() {
	load_plugin_textdomain( 'woocommerce-advanced-notifications', false, basename( WC_ADVANCED_NOTIFICATIONS_ABSPATH ) . '/languages/' );
}

/**
 * WooCommerce Deactivated Notice.
 */
function woocommerce_advanced_notifications_woocommerce_deactivated() {
	/* translators: %s: WooCommerce link */
	echo '<div class="error"><p>' . sprintf( esc_html__( 'WooCommerce Advanced Notifications requires %s to be installed and active.', 'woocommerce-advanced-notifications' ), '<a href="https://woocommerce.com/" target="_blank">WooCommerce</a>' ) . '</p></div>';
}

/**
 * Database version check.
 */
function woocommerce_advanced_notifications_db_check() {
	if ( get_option( 'wc_advanced_notifications_db_version' ) !== WC_ADVANCED_NOTIFICATIONS_DB_VERSION ) {
		activate_advanced_notifications();
	}
}

/**
 * Declaring HPOS compatibility.
 */
function woocommerce_advanced_notifications_declare_hpos_compatibility() {
	if ( class_exists( '\Automattic\WooCommerce\Utilities\FeaturesUtil' ) ) {
		\Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility( 'custom_order_tables', 'woocommerce-advanced-notifications/woocommerce-advanced-notifications.php', true );
	}
}
add_action( 'before_woocommerce_init', 'woocommerce_advanced_notifications_declare_hpos_compatibility' );

/**
 * Activation
 */
register_activation_hook( __FILE__, 'activate_advanced_notifications' );

/**
 * Advanced notifications activation function.
 */
function activate_advanced_notifications() {
	global $wpdb;

	$wpdb->hide_errors();

	$collate = '';
	if ( $wpdb->has_cap( 'collation' ) ) {
		if ( ! empty( $wpdb->charset ) ) {
			$collate .= "DEFAULT CHARACTER SET $wpdb->charset";
		}
		if ( ! empty( $wpdb->collate ) ) {
			$collate .= " COLLATE $wpdb->collate";
		}
	}

	require_once ABSPATH . 'wp-admin/includes/upgrade.php';

	/**
	 * Table for notifications
	 */
	$sql = "
CREATE TABLE {$wpdb->prefix}advanced_notifications (
notification_id bigint(20) NOT NULL auto_increment,
recipient_name LONGTEXT NULL,
recipient_email LONGTEXT NULL,
recipient_address LONGTEXT NULL,
recipient_phone varchar(240) NULL,
recipient_website varchar(240) NULL,
notification_type varchar(240) NULL,
notification_plain_text int(1) NOT NULL,
notification_totals int(1) NOT NULL,
notification_prices int(1) NOT NULL,
notification_include_all_items int(1) NULL,
notification_sent_count bigint(20) NOT NULL default 0,
PRIMARY KEY  (notification_id)
) $collate;
";
	dbDelta( $sql );

	$sql = "
CREATE TABLE {$wpdb->prefix}advanced_notification_triggers (
notification_id bigint(20) NOT NULL,
object_id bigint(20) NOT NULL,
object_type varchar(200) NOT NULL,
PRIMARY KEY  (notification_id,object_id)
) $collate;
";
	dbDelta( $sql );

	update_option( 'wc_advanced_notifications_db_version', WC_ADVANCED_NOTIFICATIONS_DB_VERSION );
}
